/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.ComposableOperatorClassPredicate;
import org.opt4j.core.Genotype;
import org.opt4j.core.optimizer.Operator;
import org.opt4j.operators.AbstractGenericOperator.OperatorPredicate;

/**
 * {@link OperatorPredicate} implementation that delegates the actual work to a
 * {@link IOperatorSelector} object. For each class of operators (e.g., mutate, crossover, ...), a
 * single {@link IOperatorSelector} is shared between operator-specific
 * {@link OperatorSelectorPredicate} wrappers.
 * 
 * @author barner
 */
public class OperatorSelectorPredicate implements OperatorPredicate {
	/** Operator whose selection is determined by this {@link OperatorSelectorPredicate}. */
	private Operator<?> operator;

	/**  */
	private ComposableOperatorClassPredicate genotypePredicate;

	/** {@link IOperatorSelector} to which the actual work is delegated. */
	private IOperatorSelector operatorSelector;

	/**
	 * Constructs a new {@link OperatorSelectorPredicate} for a given operation. All operators of a
	 * particular class (mutate, crossover, ...) must share the some {@link IOperatorSelector}
	 * object.
	 */
	public OperatorSelectorPredicate(Operator<?> operator, IOperatorSelector operatorSelector) {
		this.operator = operator;
		this.operatorSelector = operatorSelector;
		operatorSelector.register(operator);
	}

	/**
	 * Constructs a new {@link OperatorSelectorPredicate} for a given genotype(class) and operation.
	 * The genotype parameter determines the genotype to which the given operator shall be applied.
	 * All operators of a particular class (mutate, crossover, ...) must share the same
	 * {@link IOperatorSelector} object.
	 */
	public OperatorSelectorPredicate(Operator<?> operator,
			ComposableOperatorClassPredicate genotypePredicate,
			IOperatorSelector operatorSelector) {
		this.operator = operator;
		this.genotypePredicate = genotypePredicate;
		this.operatorSelector = operatorSelector;
		operatorSelector.register(operator);
	}

	/** {@inheritDoc} */
	@Override
	public boolean evaluate(Genotype genotype) {
		Boolean geneMatch = null;
		if(genotypePredicate != null) {
			geneMatch = genotypePredicate.evaluate(genotype);
		}
		boolean opMatch = operatorSelector.evaluate(operator);

		return (geneMatch != null) ? (geneMatch && opMatch) : opMatch;
	}
}
