/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem;

import java.util.Comparator;
import java.util.Set;
import java.util.TreeSet;

import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.opt4j.core.Objectives;
import org.opt4j.core.problem.Evaluator;
import org.opt4j.core.problem.MultiEvaluator;

import com.google.inject.Inject;
import com.google.inject.Provider;

/**
 * Specialized {@link MultiEvaluator} that determines the priority of the evaluators based on the
 * priorities provided in the underlying goal specification.
 * 
 * @author barner
 */
public class ExplorationGoalPriorityMultiEvaluator extends MultiEvaluator {

	/** Sorted set (order = priority, lower priority = higher priority */
	private Set<EvaluatorWrapper<?, ?, ExplorationTarget<?>>> evaluatorWrappers =
			new TreeSet<EvaluatorWrapper<?, ?, ExplorationTarget<?>>>(
					new EvaluatorWrapperPriorityComparator());

	/** Creates a new {@link ExplorationGoalPriorityMultiEvaluator}. */
	@SuppressWarnings("unchecked")
	@Inject
	public ExplorationGoalPriorityMultiEvaluator(Set<Evaluator<Object>> evaluators,
			Provider<Objectives> objectivesProvider) {
		super(evaluators, objectivesProvider);

		// Set is sorted by priority
		evaluatorWrappers
				.addAll((Set<? extends EvaluatorWrapper<?, ?, ExplorationTarget<?>>>)evaluators);
	}

	/** Priority comparator that is based on the priority provided in the goal specification */
	private static class EvaluatorWrapperPriorityComparator implements
			Comparator<EvaluatorWrapper<?, ?, ExplorationTarget<?>>> {

		/** {@inheritDoc} */
		@Override
		public int compare(EvaluatorWrapper<?, ?, ExplorationTarget<?>> e1,
				EvaluatorWrapper<?, ?, ExplorationTarget<?>> e2) {
			int p1 = e1.getPriority();
			int p2 = e2.getPriority();
			if(p1 == p2) {
				// Total order required
				return e1.hashCode() - e2.hashCode();
			}
			return p1 - p2;
		}
	}

	/** Returns the EvaluatorWrappers that are also used to validate the results. */
	public Set<EvaluatorWrapper<?, ?, ExplorationTarget<?>>> getEvaluatorWrappers() {
		return evaluatorWrappers;
	}
}
