/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.instantiatetaskgraph;

import static org.fortiss.af3.exploration.alg.util.ExplorationAlgUtils.castToRawColl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.stream.Collectors;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.AbstractTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.InstantiatedTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.exception.DecodingException;
import org.fortiss.af3.exploration.alg.dse.module.DecoderModule;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.DefaultTaskGraph;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.SignalAdapterWeightedEdge;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.TaskGraph;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.TaskGraphInstantiator;
import org.opt4j.core.Genotype;

import com.google.inject.Provides;

/**
 * {@link ComposableDecoder} that takes an {@link InstantiatedTaskMappingEncoding} as input and
 * generates an {@link InstantiatedTaskGraphEncoding} from it.
 * 
 * @author diewald
 */
public class InstantiatedTaskMappingDecoder<T extends InstantiatedTaskMappingEntry> extends
		DecoderModule<InstantiatedTaskGraphEncoding<T>> {

	@Provides
	public InstantiatedTaskGraphEncoding<FailSilentTaskMappingEntry> decode(
			FailSilentTaskMappingEncoding itmEnc, AbstractTaskGraphEncoding atgEnc,
			SafetyFunctionArchEncoding sfEnc) throws DecodingException {
		return decode(itmEnc, atgEnc, sfEnc);
	}

	public InstantiatedTaskGraphEncoding<T> decode(InstantiatedTaskMappingEncoding<T> itmEnc,
			AbstractTaskGraphEncoding atgEnc, SafetyFunctionArchEncoding sfEnc)
			throws DecodingException {
		Collection<TaskGraph> tgColl = atgEnc.getTaskGraphs();
		TaskGraphInstantiator<TaskGraph> tgInstatiator =
				new TaskGraphInstantiator<>(tgColl, itmEnc);
		InstantiatedTaskGraphEncoding<T> itgEnc =
				new InstantiatedTaskGraphEncoding<T>(
						tgInstatiator.createMultipleInstanceTaskGraphs(tgColl),
						atgEnc.getHyperPeriod());

		if(sfEnc != null) {
			removeInterSafetyChannelSignals(sfEnc, itgEnc);
		}

		return itgEnc;
	}

	/**
	 * Removes signals from the {@link InstantiatedTaskGraphEncoding} that were introduced by
	 * replications but which connect independent safety channels. This is not desired, so remove
	 * all edges (signals) in the task graph that induce connections between the safety channels.
	 */
	// FIXME: also consider other realizing components.
	// TODO: consider moving this operation
	private void removeInterSafetyChannelSignals(SafetyFunctionArchEncoding sfEnc,
			InstantiatedTaskGraphEncoding<T> itgEnc) throws DecodingException {
		for(SafetyFunctionArchEntry sfEntry : sfEnc.getAllEntries()) {
			Collection<ITaskAdapter<?>> sfCompAdps =
					castToRawColl(sfEntry.getSafetyFunctionAdapter().getRealizingComponents());
			assert (sfCompAdps.size() > 0);
			ITaskAdapter<?> sfCompAdp = sfCompAdps.stream().findAny().get();

			// Get the "containing" task graph. We must check all replacement components (instances)
			// since we don't know which one was instantiated (or it would be more costly to find
			// out).
			TaskGraph sfTG = null;
			for(ITaskAdapter<?> curSFCompAdp : sfCompAdp.getReplacementComponents()) {
				sfTG = itgEnc.getTaskGraphOf(curSFCompAdp);
				if(sfTG != null) {
					break;
				}
			}
			if(sfTG == null) {
				throw new DecodingException(sfEnc, this, "Could not retrieve the TaskGraph" +
						" of the Component Adapter " + sfCompAdp.getName() + ".");
			}

			Collection<ITaskAdapter<?>> connectedSFUnits = new ArrayList<>();
			Collection<ITaskAdapter<?>> connectedDiagUnits = new ArrayList<>();
			Collection<ITaskAdapter<?>> connectedOutUnit = new ArrayList<>();
			for(int channelIdx = 0; channelIdx < sfEntry.getNumChannels(); channelIdx++) {
				Collection<ITaskAdapter<?>> sfUnits =
						castToRawColl(sfEntry.getSafetyFunctionAdapter().getRealizingComponents());
				Collection<ITaskAdapter<?>> diagUnits =
						castToRawColl(sfEntry.getSafetyFunctionAdapter().getDiagnosticComponents());
				// TODO: knowledge about the modeL: output unit not instantiated:
				ITaskAdapter<?> outputUnit =
						sfEntry.getSafetyFunctionAdapter().getOutputComponent();
				// TODO: only for a single diag unit.
				Collection<ITaskAdapter<?>> realizingSFUnits =
						itgEnc.getTasks().stream()
								.filter(d -> sfUnits.contains(d.getReplacedComponent()))
								.collect(Collectors.toList());
				Collection<ITaskAdapter<?>> realizingDiagUnits =
						itgEnc.getTasks().stream()
								.filter(d -> diagUnits.contains(d.getReplacedComponent()))
								.collect(Collectors.toList());
				Collection<ITaskAdapter<?>> outComps =
						itgEnc.getTasks().stream()
								.filter(d -> d.getObject() == outputUnit.getObject())
								.collect(Collectors.toList());

				// Pick an sf component instance that is assigned to the current channel.
				ITaskAdapter<?> currentChannelSFUnit = null;
				for(ITaskAdapter<?> curSFUnit : realizingSFUnits) {
					if(!connectedSFUnits.contains(curSFUnit)) {
						// Remember that the diagnostic unit is associated with one channel --> no
						// elimination of the signal.
						connectedSFUnits.add(curSFUnit);
						currentChannelSFUnit = curSFUnit;
						break;
					}
				}
				// Pick an diagnostic component instance that is assigned to the current channel.
				ITaskAdapter<?> currentChannelDiagUnit = null;
				for(ITaskAdapter<?> curDiagUnit : realizingDiagUnits) {
					if(!connectedDiagUnits.contains(curDiagUnit)) {
						// Remember that the diagnostic unit is associated with one channel --> no
						// elimination of the signal.
						connectedDiagUnits.add(curDiagUnit);
						currentChannelDiagUnit = curDiagUnit;
						break;
					}
				}
				// Pick an io component instance that is assigned to the current channel.
				ITaskAdapter<?> currentChannelOutputUnit = null;
				for(ITaskAdapter<?> curOutUnit : outComps) {
					if(!connectedOutUnit.contains(curOutUnit)) {
						// Remember that the diagnostic unit is associated with one channel --> no
						// elimination of the signal.
						connectedDiagUnits.add(curOutUnit);
						currentChannelOutputUnit = curOutUnit;
						break;
					}
				}

				updateTaskGraph(sfTG, realizingSFUnits, realizingDiagUnits, outComps,
						currentChannelSFUnit, currentChannelDiagUnit, currentChannelOutputUnit);
			}
		}
	}

	/**
	 * Updates the given {@link DefaultTaskGraph} to match the number of channels.
	 * 
	 * @param sfTG
	 *            {@link TaskGraph} containing the safety function instances.
	 * @param realizingSFUnits
	 *            {@link ITaskAdapter}s realizing the safety function's protection
	 *            element.
	 * @param realizingDiagUnits
	 *            {@link ITaskAdapter}s realizing the safety function's diagnostic
	 *            element.
	 * @param outComps
	 *            {@link ITaskAdapter}s realizing the safety function's output
	 *            aggregation element, e.g., a voter.
	 * @param currentChannelSFUnit
	 *            {@link ITaskAdapter} of the currently processed protection element.
	 * @param currentChannelDiagUnit
	 *            {@link ITaskAdapter} of the currently processed diagnostic element.
	 * @param currentChannelOutputUnit
	 *            {@link ITaskAdapter} of the currently processed output aggregation
	 *            element.
	 */
	private void updateTaskGraph(TaskGraph sfTG, Collection<ITaskAdapter<?>> realizingSFUnits,
			Collection<ITaskAdapter<?>> realizingDiagUnits, Collection<ITaskAdapter<?>> outComps,
			ITaskAdapter<?> currentChannelSFUnit, ITaskAdapter<?> currentChannelDiagUnit,
			ITaskAdapter<?> currentChannelOutputUnit) {
		for(ITaskAdapter<?> curSFUnit : realizingSFUnits) {
			if(currentChannelSFUnit != curSFUnit) {
				// Eliminate all other signals to different channels.
				SignalAdapterWeightedEdge interChannelSignal =
						sfTG.getGraph().getEdge(currentChannelDiagUnit, curSFUnit);
				sfTG.getGraph().removeEdge(interChannelSignal);
				interChannelSignal = sfTG.getGraph().getEdge(currentChannelOutputUnit, curSFUnit);
				sfTG.getGraph().removeEdge(interChannelSignal);
			}
		}
		for(ITaskAdapter<?> curDiagUnit : realizingDiagUnits) {
			if(currentChannelDiagUnit != curDiagUnit) {
				// Eliminate all other signals to different channels.
				SignalAdapterWeightedEdge interChannelSignal =
						sfTG.getGraph().getEdge(currentChannelSFUnit, curDiagUnit);
				sfTG.getGraph().removeEdge(interChannelSignal);
				interChannelSignal = sfTG.getGraph().getEdge(currentChannelOutputUnit, curDiagUnit);
				sfTG.getGraph().removeEdge(interChannelSignal);
			}
		}
		for(ITaskAdapter<?> curOutUnit : outComps) {
			if(currentChannelOutputUnit != curOutUnit) {
				// Eliminate all other signals to different channels.
				SignalAdapterWeightedEdge interChannelSignal =
						sfTG.getGraph().getEdge(currentChannelSFUnit, curOutUnit);
				sfTG.getGraph().removeEdge(interChannelSignal);
				interChannelSignal = sfTG.getGraph().getEdge(currentChannelDiagUnit, curOutUnit);
				sfTG.getGraph().removeEdge(interChannelSignal);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public InstantiatedTaskGraphEncoding<T> decode(Genotype genotype) {
		// TODO Auto-generated method stub
		return null;
	}
}
