/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.dseinternal;

import static org.fortiss.af3.exploration.util.DesignSpaceExplorationModelElementFactory.createSuperSetMap;
import static org.fortiss.tooling.common.util.LambdaUtils.getFirst;

import java.util.Collection;
import java.util.Collections;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.ITransformationModule;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.base.TransformationStrategyBase;
import org.fortiss.af3.exploration.alg.guava.SubClassToInstanceMap;
import org.fortiss.af3.exploration.alg.service.ExplorationTransformationInputs;
import org.fortiss.af3.exploration.alg.service.IExplorationTransformationService.TransformationContext;
import org.fortiss.af3.exploration.dsl_v2.model.expression.SuperSet;
import org.fortiss.af3.exploration.model.SuperSetMap;

/**
 * Transforms the DSE-internal solution representations into {@link SuperSet}s and stores them in a
 * {@link SuperSetMap}.
 * The current implementation is fairly static and shall be replaced by a version built on a
 * dependency system.
 * 
 * @author diewald
 */
public class SuperSetTransformationStrategy extends TransformationStrategyBase<SuperSetMap> {
	/**
	 * Holds the generated {@link SuperSetMap} and keeps it over the different
	 * {@link org.fortiss.af3.exploration.alg.dse.modeltransformation.ITransformationModule.TransformationState}s
	 */
	SuperSetMap superSets;

	/** {@inheritDoc} */
	@Override
	public <N extends EObject, S extends EObject> SuperSetMap transform(TransformationState state,
			TransformationContext context, ExplorationTransformationInputs inputs,
			SubClassToInstanceMap<N> transformedModels, EObject outputModel) {
		if(state == TransformationState.INITIALIZING) {
			superSets = createSuperSetMap();

			ComponentExecutionUnitTransformation cToExUModule =
					(ComponentExecutionUnitTransformation)getFirst(subModules,
							m -> m instanceof ComponentExecutionUnitTransformation).get();
			superSets.put(cToExUModule.getOutputType(),
					cToExUModule.transform(state, context, inputs, transformedModels, outputModel));

			PartitionTransformation partModule = (PartitionTransformation)getFirst(subModules,
					m -> m instanceof PartitionTransformation).get();
			superSets.put(partModule.getOutputType(),
					partModule.transform(state, context, inputs, transformedModels, outputModel));
			applyAssociationUpdates(partModule);

			// Allocations depending on the partition architecture.
			TaskToPartitionTransformation tToPModule =
					(TaskToPartitionTransformation)getFirst(subModules,
							m -> m instanceof TaskToPartitionTransformation).get();
			tToPModule.applyAssociationUpdates(this);
			superSets.put(tToPModule.getOutputType(),
					tToPModule.transform(state, context, inputs, transformedModels, outputModel));

			PartitionToExecutionUnitTransformation pToExUModule =
					(PartitionToExecutionUnitTransformation)getFirst(subModules,
							m -> m instanceof PartitionToExecutionUnitTransformation).get();
			pToExUModule.applyAssociationUpdates(this);
			superSets.put(pToExUModule.getOutputType(),
					pToExUModule.transform(state, context, inputs, transformedModels, outputModel));
		}
		return superSets;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isOptional() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public TransformationState getActivationStage() {
		return TransformationState.INITIALIZING;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<? extends ITransformationModule<?>>> getParentModules() {
		return Collections.emptySet();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		return Collections.emptySet();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		return Collections.emptySet();
	}

	/** {@inheritDoc} */
	@Override
	public Class<SuperSetMap> getOutputType() {
		return SuperSetMap.class;
	}
}
