/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service;

import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.module.CreatorModule;
import org.fortiss.af3.exploration.alg.dse.module.DecoderModule;
import org.fortiss.af3.exploration.alg.dse.module.EvaluatorModule;
import org.fortiss.af3.exploration.alg.dse.module.OperatorModule;
import org.opt4j.core.Genotype;
import org.opt4j.core.optimizer.Operator;
import org.opt4j.core.problem.Creator;
import org.opt4j.core.problem.Decoder;
import org.opt4j.core.problem.Evaluator;

import com.google.inject.AbstractModule;

/**
 * Service for adding Exploration Modules to the DSE. The define the concrete problems that shall
 * be solved by the DSE process. Modules registered with this service are known to the DSE and
 * can be selected by users to define a DSE process. The Modules registered here are not defining a
 * concrete DSE process, just the concrete problems.
 * 
 * Exploration modules can be categorized into three classes: Variable definitions, Problem
 * statements and transformations, and Evaluators (solution metrics). The variable definitions
 * consist of a {@link Genotype}s (variables), {@link Operator}s (modify variables), and
 * {@link Creator}s (variable initialization). Problem statements transform {@link Genotype}s into
 * evaluable solutions implemented by {@link Decoder}s that produce {@link Phenotype}s.
 * {@link Evaluator}s
 * rate {@link Phenotype}s such that the quality of a solution can be quantified.
 * 
 * The complete DSE process is based on DI: Creators, Operators, Decoders, and Evaluators have to be
 * implemented as Guice {@link AbstractModule}s. Therefore, the package
 * {@code org.fortiss.af3.exploration.alg.dse.module} contains base classes simplifying the
 * definition of exploration modules.
 * 
 * @author diewald
 */
public interface IExplorationModuleService extends IExplorationService {

	/**
	 * Registers an optimization variable ({@code genotype}) with the DSE, along with its
	 * initializer ({@code creator}) and modificator ({@code operator}).
	 * 
	 * @param genotype
	 *            optimization variable type.
	 * @param creator
	 *            initializes the opt. variable.
	 * @param operator
	 *            modifications applied during the process iterations.
	 */
	<G extends Genotype> void registerVariable(Class<G> genotype, CreatorModule<G> creator,
			OperatorModule<G> operator);

	/**
	 * Registers a problem definition with the DSE process. The {@code phenotype} is equivalent to
	 * a candidate solution that is produced by the given {@code decoder}.
	 * 
	 * @param phenotype
	 *            candidate solution type.
	 * @param decoder
	 *            transforms variables (and/or other inputs) to candidate solutions.
	 */
	<P extends Phenotype> void registerProblem(Class<P> phenotype, DecoderModule<P> decoder);

	/**
	 * Registers the given {@code evaluator} to quantifies the quality of a solution by some metric.
	 * 
	 * @param evaluator
	 *            applies some metric to a candidate solution.
	 */
	void registerEvaluator(EvaluatorModule<Phenotype> evaluator);

	/** Returns the set of registered {@link Creator}s. */
	Set<CreatorModule<?>> getCreators();

	/** Returns the set of registered {@link Operator}s. */
	Set<OperatorModule<?>> getOperators();

	/** Returns the set of registered {@link Decoder}s. */
	Set<DecoderModule<?>> getDecoders();

	/** Returns the set of registered {@link Evaluator}s. */
	Set<EvaluatorModule<?>> getEvaluators();
}
